import ResourceAllocation from '/base-standard/ui/resource-allocation/model-resource-allocation.js';
import ActionHandler from '/core/ui/input/action-handler.js';
import Databind from '/core/ui/utilities/utilities-core-databinding.js';
import { MustGetElement, MustGetElements } from '/core/ui/utilities/utilities-dom.js';

function improveResourceAllocation() {
    const proto = Object.getPrototypeOf(ResourceAllocation);
    Object.defineProperty(proto, 'assignedResources', {
        get: function assignedResources() {
            return this._availableCities.flatMap((city) => city.currentResources);
        },
    });
    Object.defineProperty(proto, 'hasAnyResourceAssigned', {
        get: function hasAnyResourceAssigned() {
            return this.assignedResources.length > 0;
        },
    });
    proto.unassignAllResources = function () {
        // Must remove non-camels first
        this.assignedResources
            .filter((resource) => resource.type !== 'RESOURCE_CAMELS')
            .forEach((resource) => this.unassignResource(resource.value));
        const camelsInterval = setInterval(() => {
            // wait for all of the non-camel removals to be processed
            if (this.assignedResources.some((resource) => resource.type !== 'RESOURCE_CAMELS'))
                return;
            // Then the camels can go too
            this.assignedResources.forEach((resource) => this.unassignResource(resource.value));
            clearInterval(camelsInterval);
        }, 10);
    };
    proto.assignResourceToCity = function (resource, cityID) {
        const location = GameplayMap.getLocationFromIndex(resource.value);
        const args = { Location: location, City: cityID.id };
        const result = Game.PlayerOperations.canStart(GameContext.localPlayerID, PlayerOperationTypes.ASSIGN_RESOURCE, args, false);
        if (result.Success) {
            Game.PlayerOperations.sendRequest(GameContext.localPlayerID, PlayerOperationTypes.ASSIGN_RESOURCE, args);
        }
    };
    proto.hasFactoryResourceSelected = function () {
        if (!this._selectedResource)
            return false;
        const factoryResource = this.availableFactoryResources.find((resource) => resource.value == this.selectedResource);
        return (factoryResource &&
            factoryResource.classType === 'RESOURCECLASS_FACTORY');
    };
    proto.fillCityWithFactoryResource = function (cityID) {
        const city = this.availableCities.find((city) => city.id.id == cityID);
        if (!city)
            return false;
        const factoryResource = this.availableFactoryResources.find((resource) => resource.value == this.selectedResource);
        if (!factoryResource ||
            factoryResource.classType !== 'RESOURCECLASS_FACTORY')
            return false;
        const matchingAvailableFactoryResources = this.availableFactoryResources.filter((resource) => resource.name === factoryResource.name);
        const numSlotsToFill = Math.min(matchingAvailableFactoryResources.length, city.emptySlots.length);
        const resourcesToAssign = matchingAvailableFactoryResources.slice(0, numSlotsToFill);
        resourcesToAssign.forEach((resource) => this.assignResourceToCity(resource, city.id));
        this.clearSelectedResource();
        return true;
    };
    proto.unassignAllResourceInstancesFromCity = function (cityID, resourceValue) {
        const city = this.availableCities.find((city) => city.id.id == cityID);
        if (!city)
            return;
        const factoryResource = city.currentResources.find((resource) => resource.value == resourceValue);
        if (!factoryResource)
            return;
        const matchingAssignedFactoryResources = city.currentResources.filter((resource) => resource.name === factoryResource.name &&
            resource.value !== factoryResource.value);
        matchingAssignedFactoryResources.forEach((resource) => this.unassignResource(resource.value));
    };
    Object.defineProperty(proto, 'hasFactories', {
        get: function hasFactories() {
            return this.availableCities.some((city) => city.hasFactory);
        },
    });
    Object.defineProperty(proto, 'playerCities', {
        get: function playerCities() {
            if (this._playerCities)
                return this._playerCities;
            const localPlayerID = GameContext.localPlayerID;
            const localPlayer = Players.get(localPlayerID);
            if (!localPlayer) {
                console.error(`model-resource-allocation: Failed to retrieve PlayerLibrary for Player ${localPlayerID}`);
                return undefined;
            }
            const playerCities = localPlayer.Cities;
            if (!playerCities) {
                console.error(`model-resource-allocation: Failed to retrieve Cities for Player ${localPlayerID}`);
                return undefined;
            }
            this._playerCities = playerCities;
            return playerCities;
        },
    });
    Object.defineProperty(proto, 'distantLandsCityIds', {
        get: function distantLandsCityIds() {
            return this.playerCities
                .getCityIds()
                .filter((cityID) => Cities.get(cityID)?.isDistantLands)
                .map((cityID) => cityID.id);
        },
    });
    Object.defineProperty(proto, 'hasDistantLandsSettlements', {
        get: function hasDistantLandsSettlements() {
            return this.distantLandsCityIds.length > 0;
        },
    });
    proto.isDistantLandsCity = function (cityID) {
        return this.distantLandsCityIds.some((id) => id == cityID);
    };
}

const ExtendedResourceAllocation = ResourceAllocation;

const AllResourcesUnassignedEventName = 'all-resources-unassigned';
class AllResourcesUnassignedEvent extends CustomEvent {
    constructor() {
        super(AllResourcesUnassignedEventName, { bubbles: true });
    }
}

class UnassignAllResourcesButton extends Component {
    unassignAllResourcesActivateListener = this.onUnassignAllResourcesActivate.bind(this);
    onInitialize() {
        super.onInitialize();
        const activatable = MustGetElement('.unassign-all-resources', this.Root);
        activatable.addEventListener('action-activate', this.unassignAllResourcesActivateListener);
        const button = MustGetElement('.unassign-all-button', this.Root);
        Databind.classToggle(button, 'hidden', '!{{g_ResourceAllocationModel.hasAnyResourceAssigned}} || {{g_ResourceAllocationModel.isResourceAssignmentLocked}}');
    }
    onUnassignAllResourcesActivate(event) {
        if (!(event.target instanceof HTMLElement)) {
            return;
        }
        if (ActionHandler.isGamepadActive) {
            return;
        }
        ExtendedResourceAllocation.unassignAllResources();
        this.Root.dispatchEvent(new AllResourcesUnassignedEvent());
    }
}
Controls.define('unassign-all-resources-button', {
    createInstance: UnassignAllResourcesButton,
    description: 'Button to unassign all resources',
    styles: [
        'fs://game/tbq-resource-allocation-improvements/src/resource-allocation/button/unassign-all-resources-button.css',
    ],
    content: [
        'fs://game/tbq-resource-allocation-improvements/src/resource-allocation/button/unassign-all-resources-button.html',
    ],
});

class ScreenResourceAllocationDecorator {
    assignedResourceUnassignListener;
    allResourcesUnassignedListener;
    factoryResourceAssignedListener;
    factoryResourceUnassignedListener;
    screen;
    onShowTownsChanged;
    onShowCitiesChanged;
    onShowFactoriesChanged;
    onShowDistantLandsChanged;
    Root;
    showingTowns = true;
    showingCities = true;
    showingOnlyFactories = false;
    showingOnlyDistantLands = false;
    constructor(val) {
        this.screen = val;
        this.Root = this.screen.Root;
        this.assignedResourceUnassignListener =
            this.onAssignedResourceUnassign.bind(this);
        this.allResourcesUnassignedListener =
            this.onAllResourcesUnassigned.bind(this);
        this.factoryResourceAssignedListener =
            this.onFactoryResourceAssigned.bind(this);
        this.factoryResourceUnassignedListener =
            this.onFactoryResourceUnassigned.bind(this);
    }
    beforeAttach() { }
    afterAttach() {
        this.setupCityFilter();
        this.setupTownFilter();
        this.setupFactoryFilter();
        this.setupDistantLandsFilter();
        this.addFilterCategoryLabels();
        this.removeSettlementTypeNames();
        this.addDistantLandsIcons();
        this.setupQuickResourceUnassignment();
        this.addUnassignAllButton();
        this.enhanceFactorySlots();
    }
    beforeDetach() { }
    afterDetach() {
        this.Root.removeEventListener(AllResourcesUnassignedEventName, this.allResourcesUnassignedListener);
    }
    onAttributeChanged(name, prev, next) { }
    toggleSettlementVisibilities() {
        this.Root.querySelectorAll('.city-outer').forEach((cityOuter) => {
            const settlementType = cityOuter.getAttribute('settlement-type');
            const isCity = settlementType !== 'Town';
            const hasFactory = cityOuter.getAttribute('has-factory') === 'true';
            const cityEntry = cityOuter.querySelector('.city-entry');
            const cityId = cityEntry?.getAttribute('data-city-id');
            const isDistantLands = ExtendedResourceAllocation.isDistantLandsCity(cityId);
            const isSettlementSelectedByType = (isCity && this.showingCities) ||
                (!isCity && this.showingTowns);
            let isSettlementVisible = this.showingOnlyFactories
                ? isSettlementSelectedByType && hasFactory
                : isSettlementSelectedByType;
            isSettlementVisible = this.showingOnlyDistantLands
                ? isSettlementVisible && isDistantLands
                : isSettlementVisible;
            cityOuter.classList.toggle('hidden', !isSettlementVisible);
        });
    }
    setupFactoryFilter() {
        this.onShowFactoriesChanged = (event) => {
            this.showingOnlyFactories = event.detail.value;
            this.toggleSettlementVisibilities();
        };
        const showFactories = MustGetElement('.show-factories', this.Root);
        showFactories.removeEventListener('component-value-changed', this.screen.onShowFactoriesChanged);
        showFactories.setAttribute('selected', 'false');
        showFactories.previousSibling.setAttribute('data-l10n-id', 'LOC_UI_RESOURCE_ALLOCATION_SHOW_ONLY_FACTORIES');
        showFactories.addEventListener('component-value-changed', this.onShowFactoriesChanged);
    }
    setupDistantLandsFilter() {
        this.onShowDistantLandsChanged = (event) => {
            this.showingOnlyDistantLands = event.detail.value;
            this.toggleSettlementVisibilities();
        };
        const showDistantLandsContainer = document.createElement('div');
        showDistantLandsContainer.className =
            'show-distant-lands relative flex items-end mr-3';
        const showDistantLandsLabel = document.createElement('div');
        showDistantLandsLabel.className = 'font-body text-xs mb-1';
        showDistantLandsLabel.setAttribute('data-l10n-id', 'LOC_UI_RESOURCE_ALLOCATION_SHOW_DISTANT_LANDS');
        showDistantLandsLabel.setAttribute('selected', 'false');
        const checkbox = document.createElement('fxs-checkbox');
        checkbox.className = 'show-distant-lands';
        checkbox.setAttribute('selected', 'false');
        checkbox.addEventListener('component-value-changed', this.onShowDistantLandsChanged);
        showDistantLandsContainer.appendChild(showDistantLandsLabel);
        showDistantLandsContainer.appendChild(checkbox);
        const cityFilterContainer = MustGetElement('.city-filter-container', this.Root);
        if (!ExtendedResourceAllocation.hasDistantLandsSettlements) {
            showDistantLandsContainer.classList.add('hidden');
        }
        const showFactories = MustGetElement('.show-factories', this.Root);
        cityFilterContainer.insertBefore(showDistantLandsContainer, showFactories.parentNode);
    }
    setupCityFilter() {
        this.onShowCitiesChanged = (event) => {
            this.showingCities = event.detail.value;
            this.toggleSettlementVisibilities();
        };
        const showCities = MustGetElement('.show-cities', this.Root);
        showCities.removeEventListener('component-value-changed', this.screen.onShowTownsChanged);
        showCities.previousSibling.setAttribute('data-l10n-id', 'LOC_UI_RESOURCE_ALLOCATION_SHOW_CITIES');
        showCities.addEventListener('component-value-changed', this.onShowCitiesChanged);
    }
    setupTownFilter() {
        this.onShowTownsChanged = (event) => {
            this.showingTowns = event.detail.value;
            this.toggleSettlementVisibilities();
        };
        const showTownsContainer = document.createElement('div');
        showTownsContainer.className = 'relative flex items-end mr-3';
        const showTownsLabel = document.createElement('div');
        showTownsLabel.className = 'font-body text-xs mb-1';
        showTownsLabel.setAttribute('data-l10n-id', 'LOC_UI_RESOURCE_ALLOCATION_SHOW_TOWNS');
        showTownsLabel.setAttribute('selected', 'true');
        const checkbox = document.createElement('fxs-checkbox');
        checkbox.className = 'show-towns';
        checkbox.setAttribute('selected', 'true');
        checkbox.addEventListener('component-value-changed', this.onShowTownsChanged);
        showTownsContainer.appendChild(showTownsLabel);
        showTownsContainer.appendChild(checkbox);
        const cityFilterContainer = MustGetElement('.city-filter-container', this.Root);
        const showCities = MustGetElement('.show-cities', this.Root);
        cityFilterContainer.insertBefore(showTownsContainer, showCities.parentNode.nextSibling);
    }
    createFilterCategoryLabel(locId) {
        const label = document.createElement('p');
        label.className =
            'city-column-header flex justify-start font-title text-base uppercase text-secondary relative';
        label.style.paddingLeft = '8px';
        label.style.paddingRight = '12px';
        label.style.top = '-4px';
        label.textContent = `${Locale.compose(locId)}:`;
        return label;
    }
    createSeparator() {
        const separator = document.createElement('div');
        separator.textContent = '|';
        separator.className = 'text-xl font-title text-secondary relative';
        separator.style.paddingLeft = '6px';
        separator.style.paddingRight = '6px';
        separator.style.top = '2px';
        return separator;
    }
    addFilterCategoryLabels() {
        const showYields = MustGetElement('.show-yields', this.Root);
        const showCities = MustGetElement('.show-cities', this.Root);
        const showTowns = MustGetElement('.show-towns', this.Root);
        MustGetElement('.show-factories', this.Root);
        showYields.previousSibling.setAttribute('data-l10n-id', 'LOC_RESOURCE_FILTER_YIELDS');
        showCities.previousSibling.setAttribute('data-l10n-id', 'LOC_UI_SETTLEMENT_TAB_BAR_CITIES');
        showTowns.previousSibling.setAttribute('data-l10n-id', 'LOC_UI_SETTLEMENT_TAB_BAR_TOWNS');
        const uiLabel = this.createFilterCategoryLabel('LOC_UI_RESOURCE_ALLOCATION_UI_LABEL');
        showYields.parentElement?.insertAdjacentElement('beforebegin', uiLabel);
        const showLabel = this.createFilterCategoryLabel('LOC_RESOURCE_FILTER_SHOW');
        showCities.parentElement?.insertAdjacentElement('beforebegin', this.createSeparator());
        showCities.parentElement?.insertAdjacentElement('beforebegin', showLabel);
        const filterLabel = this.createFilterCategoryLabel('LOC_ADVANCED_START_FILTER');
        const separator = this.createSeparator();
        if (!(ExtendedResourceAllocation.hasDistantLandsSettlements ||
            ExtendedResourceAllocation.hasFactories)) {
            filterLabel.classList.add('hidden');
            separator.classList.add('hidden');
        }
        showTowns.parentElement?.insertAdjacentElement('afterend', filterLabel);
        showTowns.parentElement?.insertAdjacentElement('afterend', separator);
    }
    removeSettlementTypeNames() {
        const settlementTypeNames = MustGetElements('.settlement-type-text', this.Root);
        settlementTypeNames.forEach((settlementTypeName) => (settlementTypeName.style.visibility = 'hidden'));
    }
    addDistantLandsIcons() {
        // Wait for the city IDs to be data bound
        const waitForCityId = new Promise((resolve) => {
            const waitForCityIdHandler = setInterval(() => {
                const cityEntries = this.Root.querySelectorAll('.city-entry');
                const cityID = cityEntries[0].getAttribute('data-city-id');
                if (cityID) {
                    clearInterval(waitForCityIdHandler);
                    resolve();
                }
            }, 20);
        });
        const distantLandsIcon = `<div style="padding-left: 10px;">
                                    <fxs-icon class="size-8 bg-no-repeat bg-center" data-icon-id="NOTIFICATION_DISCOVER_CONTINENT"></fxs-icon>
                                  </div>`;
        waitForCityId.then(() => {
            this.Root.querySelectorAll('.city-entry').forEach((cityEntry) => {
                const settlementName = MustGetElement('.settlement-name-text', cityEntry);
                const cityID = cityEntry.getAttribute('data-city-id');
                const isDistantLands = ExtendedResourceAllocation.isDistantLandsCity(cityID);
                if (isDistantLands) {
                    settlementName.insertAdjacentHTML('afterend', distantLandsIcon);
                }
            });
        });
    }
    setupQuickResourceUnassignment() {
        const cityResources = MustGetElements('.city-resource', this.Root);
        cityResources.forEach((cityResource) => cityResource.addEventListener('auxclick', this.assignedResourceUnassignListener));
    }
    addUnassignAllButton() {
        const cityHeaderContainer = MustGetElement('.city-header-container', this.Root);
        const cityFilterContainer = MustGetElement('.city-filter-container', this.Root);
        cityHeaderContainer.classList.remove('grow');
        cityFilterContainer.classList.remove('grow');
        cityHeaderContainer.insertAdjacentHTML('afterend', `<div class="flex grow w-100 justify-end mr-10">
                <unassign-all-resources-button></unassign-all-resources-button>
            </div>`);
        this.Root.addEventListener(AllResourcesUnassignedEventName, this.allResourcesUnassignedListener);
    }
    onAssignedResourceUnassign(event) {
        if (!(event.target instanceof HTMLElement)) {
            return;
        }
        if (ActionHandler.isGamepadActive) {
            return;
        }
        const resourceValueAttribute = event.target.getAttribute('data-resource-value');
        const resourceClassAttribute = event.target.getAttribute('data-resource-class');
        if (!resourceValueAttribute || !resourceClassAttribute) {
            console.error('screen-resource-allocation: onAssignedResourceActivate(): Failed to get attributes for resource!');
            return;
        }
        const resourceValue = parseInt(resourceValueAttribute);
        ResourceAllocation.unassignResource(resourceValue);
        this.refreshStates();
    }
    onAllResourcesUnassigned(event) {
        this.refreshStates();
    }
    refreshStates() {
        this.screen.updateCityEntriesDisabledState();
        this.screen.updateAvailableResourceColDisabledState();
        this.screen.updateAllUnassignActivatable();
        this.screen.focusCityList();
    }
    enhanceFactorySlots() {
        let factorySlots;
        try {
            factorySlots = MustGetElements('.city-factory-resource-container > fxs-activatable');
        }
        catch (error) {
            // no factory slots found
            return;
        }
        const containers = MustGetElements('.city-factory-resource-container');
        containers.forEach((container) => {
            const emptySlot = MustGetElement('fxs-activatable', container);
            emptySlot.setAttribute('data-bind-class-toggle', `bg-accent-4:{{g_ResourceAllocationModel}}.hasFactoryResourceSelected();opacity-70:(!!{{g_ResourceAllocationModel.selectedResourceClass}}&&!{{g_ResourceAllocationModel}}.hasfactoryResourceSelected())`);
        });
        factorySlots.forEach((slot) => slot.removeEventListener('action-activate', this.screen.cityActivateListener));
        factorySlots.forEach((slot) => slot.addEventListener('action-activate', this.factoryResourceAssignedListener));
        factorySlots.forEach((slot) => slot.addEventListener('auxclick', this.factoryResourceUnassignedListener));
    }
    onFactoryResourceAssigned(event) {
        const resourceValue = event.target.getAttribute('data-resource-value');
        if (!resourceValue) {
            // empty factory slot
            if (ResourceAllocation.hasSelectedResource()) {
                const cityID = event.target.parentElement?.getAttribute('data-city-id');
                const isSuccess = ExtendedResourceAllocation.fillCityWithFactoryResource(cityID);
                if (isSuccess)
                    return;
            }
        }
        // fallback to default behavior
        this.screen.onCityActivate(event);
        return;
    }
    onFactoryResourceUnassigned(event) {
        const resourceValue = event.target.getAttribute('data-resource-value');
        const resourceClass = event.target.getAttribute('data-resource-class');
        if (!resourceValue || resourceClass !== 'RESOURCECLASS_FACTORY')
            return;
        const cityID = event.target.parentElement?.getAttribute('data-city-id');
        ExtendedResourceAllocation.unassignAllResourceInstancesFromCity(cityID, resourceValue);
    }
}
Controls.decorate('screen-resource-allocation', (val) => new ScreenResourceAllocationDecorator(val));

function initialize() {
    improveResourceAllocation();
}
Loading.runWhenFinished(initialize);
